#!/usr/bin/env python
"""environmentutil.py
Utility tool helps to manage mappings and gathering data from across multiple AWS Availability zones.

Usage:
    environmentutil amimap get [--aws_region <AWS_REGION>]
            [--config_file <CONFIG_FILE>]
    environmentutil amimap write [<OUTPUT_FILE>] [--aws_region <AWS_REGION>]
            [--config_file <CONFIG_FILE>]
    environmentutil deploy <CLOUDFORMATION_TEMPLATE> [<PARAMETER_JSON_FILE>]
            [--aws_region <AWS_REGION>] [--config_file <CONFIG_FILE>]

Options:
  -h --help                    Show this screen.
  -v --version                 Show version.
  --aws_region <AWS_REGION>    Region to start queries to AWS API from [default: us-east-1].
  --config_file <CONFIG_FILE>  JSON Config file holding the extended configuration for this toolset [default: config_args.json].
"""

from docopt import docopt
import boto
import json
import logging
from .version import __version__
import time


class EnvironmentUtil(object):
    """
    EnvironmentUtil class holds common task methods for deploying, managing or
    building CloudFormation templates with the environmenbase toolset.
    """

    def __init__(self,
                 config_args):
        """
        Init for EnvironmentUtil class which persists config args as a dictionary
        @param config_args [dict] - dictionary of configuration values
        """
        self.configuration = config_args

    def get_ami_map(self,
                    aws_region=None, image_names=None):
        """
        Method iterates on all AWS regions for a given set of AMI names to gather AMI IDs and
        to create a regionmap for CloudFormation templates.
        @param aws_region [string] - optionally provides the region to start querying when gathering the list of regions globally.
        """
        if aws_region is None:
            aws_region = self.configuration.get('boto', {}).get('region_name', 'us-east-1')
            logging.debug('Setting default AWS Region for API access from overall configuration [' + aws_region + ']')
        region_map = {}
        vpc_conn = boto.connect_vpc(aws_region)
        logging.debug('Connected to VPC in region [' + aws_region + ']')
        for region in vpc_conn.get_all_regions():
            if region.name not in region_map.keys():
                logging.debug('Adding region [' + region.name + '] to region map.')
                region_map[region.name] = {}
            ec2_conn = boto.connect_ec2(region.name)
            logging.debug('Connected to EC2 API in region [' + region.name + ']')
            for k, v in image_names:
                logging.debug('Looking for Image [' + k + ': ' + v + '] in region [' + region.name + ']')
                images = ec2_conn.get_all_images(filters={'name': v})
                if len(images) == 0:
                    logging.warn('No image found for [' + k + ': ' + v + '] in region [' + region.name + ']')
                elif len(images) > 1:
                    logging.warn('Found ' + str(len(images)) + ' images for [' + k + ': ' + v + '] in region [' + region.name + ']')
                else:
                    logging.debug('Adding image [' + images[0].id + '] to region [' + region.name + '] for key [' + k + ']')
                    region_map[region.name][k] = images[0].id
        logging.debug('AMI Region Map Contents: ' + json.dumps(region_map))
        return region_map

    def write_ami_map(self,
                      aws_region,
                      output_file):
        """
        Utility and convenience method for wrapping the get_ami_map method and subsequently
        writing the output to a file for use as an ami id cache.
        @param aws_region [string] - AWS-specific region name to start when querying the AWS APIs
        @param output_file [string] - file location where the ami cache is to be saved locally
        """
        with open(output_file, 'w') as f:
            logging.debug('Writing ami cache file to [' + output_file + ']')
            f.write(json.dumps(self.get_ami_map(aws_region)))

    def get_stack_status(self,
                         cf_conn,
                         stack_name):
        """
        Helper method handles edge cases when stack status doesn't exist yet or any more.
        @param cf_conn [Boto.CloudFormation.Connection] - Connection object to CloudFormation via Boto
        @param stack_name [string] - Name of the stack to check status on
        """
        api_result = cf_conn.describe_stacks(stack_name_or_id=stack_name)
        if len(api_result) == 0:
            return 'NOT_CREATED'
        else:
            return api_result[0].stack_status

    def wait_for_stack(self,
                       cf_conn,
                       stack_name,
                       sleep_time=20):
        """
        Method handles a wait loop for stack deploys to AWS. Sleep time should be ramped up (longer polls)
        when deploying multiple sets of stacks at the same time.
        Returns true when deploy is successful, false when errors occur.
        @param cf_conn [Boto.CloudFormation.Connection] - Connection object to CloudFormation via Boto
        @param stack_name [string] - Name of the stack to check status on
        @param sleep_time [int] - number of seconds to wait between polls of the AWS API for status on the specified CloudFormation stack
        """
        stack_status = self.get_stack_status(cf_conn, stack_name)
        loop_id = 0
        while 'IN_PROGRESS' in stack_status:
            if loop_id != 0:
                message = 'Stack %s is not yet completely deployed. Waiting 20 sec until next polling interval. ' \
                          'Update query count [ %s ]' % (stack_name, str(loop_id))
                logging.info(message)
                time.sleep(sleep_time)
            stack_status = self.get_stack_status(cf_conn, stack_name)
            loop_id += 1

        if cf_conn.describe_stacks(stack_name_or_id=stack_name)[0].stack_status in ['CREATE_COMPLETE', 'UPDATE_COMPLETE']:
            return True
        else:
            return False

    def deploy_stack(self,
                     stack_name,
                     template_string_or_url,
                     capabilities=['CAPABILITY_IAM'],
                     parameters=None,
                     aws_region=None,
                     wait_for_complete=True):
        """
        Method takes a CloudFormation template string or S3 url and deploys the stack to the specified AWS region.
        @param stack_name [string] - name to use when deploying the CloudFormation stack.
        @param template_string_or_url [string] - S3 URL or CloudFormation template body to be deployed.
        @param capabiltiies [list(str)] - List of CloudFormation template capabilities to be granted to the deployed stack.
        @param parameters [dict] - dictionary of key value pairs containing overrides to template parameter defaults.
        @param aws_region [string] - AWS-specific region name to start when querying the AWS APIs
        @param wait_for_complete [boolean] - boolean indicating whether to poll for success or failure before completing the deploy process.
        """
        if aws_region is None:
            aws_region = self.configuration.get('boto', {}).get('region_name', 'us-east-1')
            logging.debug('Setting default AWS Region for API access from overall configuration [' + aws_region + ']')

        logging.info('Connecting to CloudFormation in region [' + aws_region + ']')
        cf_conn = boto.connect_cloudformation(aws_region)
        logging.info('Starting deploy of stack [' + stack_name + '] to AWS in region [' + aws_region + ']')

        command_args = {'capabilities': capabilities}

        try:
            if type(template_string_or_url) == dict:
                command_args['template_body'] = json.dumps(template_string_or_url)
            else:
                # template_dict = json.loads(template_string_or_url)
                command_args['template_body'] = template_string_or_url
        except:
            command_args['template_s3_url'] = template_string_or_url

        logging.debug('Calling stack deploy for [' + stack_name + '] with arguments: ' + json.dumps(command_args))
        cf_conn.create_stack(stack_name, **command_args)

        if wait_for_complete:
            if self.wait_for_stack(cf_conn, stack_name):
                logging.info('Stack [' + stack_name + '] successfully deployed to AWS in region [' + aws_region + ']')
                return True
            else:
                message = 'Stack [%s] failed to deploy to AWS in region [%s] with status [%s]' % (
                    stack_name, aws_region,
                    self.get_stack_status(cf_conn, stack_name))
                logging.warn(message)
                return False
        else:
            return True


if __name__ == '__main__':
    arguments = docopt(__doc__, version='environmentbase-cfn environment_util %s' % __version__)

    if arguments.get('--debug', False):
        level = 'DEBUG'
    else:
        level = 'INFO'
    logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s', level=level)

    config_file_path = arguments.get('--config_file', 'config_args.json')
    if config_file_path:
        with open(config_file_path, 'r') as f:
            logging.info('Reading configuration in from ')
            json_data = json.loads(f.read())
    else:
        json_data = {}

    if arguments.get('amimap'):
        env_util = EnvironmentUtil(json_data)

        if arguments.get('get'):
            logging.info('Getting AMI Map and printing to console.')
            logging.info(
                json.dumps(
                    env_util.get_ami_map(arguments.get('--aws_region', 'us-east-1'))))
        elif arguments.get('write'):
            file_location = arguments.get('<OUTPUT_FILE>', 'ami_cache.json')
            logging.info('Getting AMI Map and writing to file [' + file_location + ']')
            env_util.write_ami_map(arguments.get('--aws_region', 'us-east-1'), file_location)
    elif arguments.get('deploy'):
        pass



from environmentbase.template import Template
from troposphere import Ref, Join, Output, GetAtt, AWS_ACCOUNT_ID
from troposphere.cloudfront import Distribution, DistributionConfig, Origin, S3Origin
from troposphere.cloudfront import DefaultCacheBehavior, ForwardedValues, Logging


class CloudFront(Template):
    """
    Creates a CloudFront distribution from a static resource
    """

    def __init__(self, resource_name, domain_name, origin_path='', dist_config=None):
        """
        This will create a cloudfront distribution from a static resource
        @param resource_name [string] - name of the cloudfront distribution to be created
        @param domain_name [string] - domain name of the s3 bucket containing the resource
        @param origin_path [string] - path to the folder containing the resources (optional)
        """

        self.resource_name = resource_name
        self.domain_name = domain_name
        self.origin_path = origin_path
        self.dist_config = dist_config

        super(CloudFront, self).__init__(template_name=resource_name)

    def build_hook(self):
        """
        Hook to add tier-specific assets within the build stage of initializing this class.
        """

        if not self.dist_config:
            self.dist_config = DistributionConfig(
                Origins=[Origin(
                    Id="Origin",
                    DomainName=self.domain_name,
                    OriginPath=self.origin_path,
                    S3OriginConfig=S3Origin(),
                )],
                DefaultCacheBehavior=DefaultCacheBehavior(
                    TargetOriginId="Origin",
                    ForwardedValues=ForwardedValues(
                        QueryString=False
                    ),
                    ViewerProtocolPolicy="allow-all"),
                Enabled=True
            )

        if self.utility_bucket:
            self.dist_config.Logging = Logging(
                Bucket=Join('.', [self.utility_bucket, 's3.amazonaws.com']),
                IncludeCookies=True,
                Prefix=Join('/', ['AWSLogs', Ref(AWS_ACCOUNT_ID), 'CloudFront'])
            )

        cf_distribution = self.add_resource(Distribution(
            self.resource_name,
            DistributionConfig=self.dist_config
        ))

        self.add_output([
            Output("DistributionId", Value=Ref(cf_distribution)),
            Output("DistributionName", Value=Join("", ["http://", GetAtt(cf_distribution, "DomainName")])),
        ])



# -*- encoding: utf8 -*-
import io
from os.path import dirname
from os.path import join
from setuptools import find_packages
from setuptools import setup
from multiprocessing import util

exec(open('src/environmentbase/version.py').read())


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ).read()

setup(
    name="cfn-environment-base",

    # Version is centrally managed from src/environmentbase/version.py
    version=__version__,

    description="Base environment for Troposphere based CFN project environments",
    long_description="%s" % read("README.rst"),

    url='https://github.com/AWSFrederick/cloudformation-environmentbase',

    author="Patrick McClory, Patrick Pierson",
    author_email="patrick@dualspark.com, patrick.pierson@ionchannel.io",

    license="ISC",

    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Utilities",
        "Topic :: Software Development :: Code Generators"
    ],

    # keywords=["keyword1", "keyword2", "keyword3"],

    # List out the packages to include when running build/install/distribute
    packages=find_packages("src", exclude=['tests*']),

    # For more fine grain control over modules included or excluded use py_modules
    # py_modules=[splitext(basename(i))[0] for i in glob.glob("src/**/*.py")],

    # Specifies the root/default package is below 'src'
    package_dir={"": "src"},

    install_requires=[
        "troposphere==1.9.3",
        "jmespath==0.9.3",
        "boto==2.47.0",
        "botocore==1.5.56",
        "boto3==1.4.4",
        "ipcalc==1.99.0",
        "docopt==0.6.2",
        "setuptools==35.0.2",
        "awacs==0.6.2",
        "commentjson==0.6",
        "PyYAML==3.12",
        "netaddr==0.7.19",
        "toolz==0.8.2",
        'pbr==3.0.1',
        'six==1.10.0'
    ],

    # Optional dependencies
    extras_require={},

    # This section is required for setuptools to auto-gen the cross platform wrapper script
    # i.e. 'environmentbase --version' instead of 'python -m environmentbase --version'
    entry_points={
        "console_scripts": [
            "environmentbase = environmentbase.__main__:main",
            # "environmentutil = environmentutil.environmentutil:main",
            # "awsbootstrap = environmentbase.accountbootstrap:main"
        ]
    },

    # If disabled, generated egg will only contain *.py files.
    # Leave enabled so we can distribute factory default data w/in the packages
    include_package_data=True,

    # Enable if the package can run as a zip file (some performance improvements)
    # Disable if it needs to run as an extracted zip inside <python>/site-packages
    # Requires special resource handling for embedded data files, see:
    # http://peak.telecommunity.com/DevCenter/PythonEggs#accessing-package-resources
    zip_safe=True,

    # Test runner and required testing packages
    test_suite='nose2.collector.collector',
    tests_require=[
        'nose2==0.5.0',
        'unittest2==1.1.0',
        'mock==1.3.0'
    ]
)



import os
import os.path
import copy
import botocore.exceptions
from boto import cloudformation, sts
from troposphere import Parameter
from .template import Template
from . import cli
from . import resources as res
from fnmatch import fnmatch
from . import utility
from . import monitor
import json
import six

TIMEOUT = 60


class ValidationError(Exception):
    pass


class EnvConfig(object):

    def __init__(self, config_handlers=None):
        self.config_handlers = config_handlers if config_handlers else []
        # self.stack_event_handlers = stack_event_handlers if stack_event_handlers else []
        # self.deploy_handlers = deploy_handlers if deploy_handlers else {}


class EnvironmentBase(object):
    """
    EnvironmentBase encapsulates functionality required to build and deploy a network and common resources for object storage within a
     specified region
    """

    def __init__(self,
                 view=None,
                 env_config=EnvConfig(),
                 config_filename=(res.DEFAULT_CONFIG_FILENAME + res.EXTENSIONS[0]),
                 config_file_override=None):
        """
        Init method for environment base creates all common objects for a given environment within the CloudFormation
        template including a network, s3 bucket and requisite policies to allow ELB Access log aggregation and
        CloudTrail log storage.
        :param view: View object to use.
        :param create_missing_files: Specifies policy to use when local files are missing.  When disabled missing files will cause an
        IOException
        :param config_filename: The name of the config file to load by default.  Note: User can still override this value from the CLI with
         '--config-file'.
        :param config: Override loading config values from file by providing config setting directly to the constructor
        """

        self.config_filename = config_filename
        self.env_config = env_config
        self.config_file_override = config_file_override
        self.config = {}
        self.globals = {}
        self.template_args = {}
        self.template = None
        self.deploy_parameter_bindings = []
        self.ignore_outputs = ['templateValidationHash', 'dateGenerated']
        self.stack_outputs = {}
        self._config_handlers = []
        self.stack_monitor = None
        self._ami_cache = None
        self.cfn_connection = None
        self.sts_credentials = None

        self.boto_session = None

        # self.env_config = env_config
        for config_handler in env_config.config_handlers:
            self._add_config_handler(config_handler)
        self.add_config_hook()

        # Load the user interface
        self.view = view if view else cli.CLI()

        # The view may override the config file location (i.e. command line arguments)
        if hasattr(self.view, 'config_filename') and self.view.config_filename is not None:
            self.config_filename = self.view.config_filename

        # Allow the view to execute the user's requested action
        self.view.process_request(self)

    def create_hook(self):
        """
        Override in your subclass for custom resource creation.  Called after config is loaded and template is
        initialized.  After the hook completes the templates are serialized and written to file and uploaded to S3.
        """
        pass

    def add_config_hook(self):
        """
        Override in your subclass for adding custom config handlers.
        Called after the other config handlers have been added.
        After the hook completes the view is loaded and started.
        """
        pass

    def deploy_hook(self):
        """
        Extension point for modifying behavior of deploy action. Called after config is loaded and before
        cloudformation deploy_stack is called. Some things you can do in deploy_hook include modifying
        config or deploy_parameter_bindings or run arbitrary commands with boto.
        """
        pass

    def delete_hook(self):
        """
        Extension point for modifying behavior of delete action. Called after config is loaded and before cloudformation
        deploy_stack is called. Can be used to manage out-of-band resources with boto.
        """
        pass

    def stack_event_hook_wrapper(self, event_data):
        """
        Write the stack outputs to file before calling the stack_event_hook that the user overrides
        """
        if self.config.get('global').get('write_stack_outputs'):
            self.write_stack_outputs_to_file(event_data)
        self.stack_event_hook(event_data)

    def stack_event_hook(self, event_data):
        """
        Extension point for reacting to the cloudformation stack event stream.  If global.monitor_stack is enabled in
        config this function is used to react to stack events. Once a stack is created a notification topic will begin
        emitting events to a queue.  Each event is passed to this call for further processing. Details about the event
        data can be read here:
        http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-listing-event-history.html
        :param event_data: The event_data hash provided the following mappings from the raw cloudformation event:
            "status" = "ResourceStatus"
            "type"   = "ResourceType"
            "name"   = "LogicalResourceId"
            "id"     = "PhysicalResourceId"
            "reason" = "ResourceStatusReason"
            "props"  = "ResourceProperties"
        :return bool: Indicates that processing is complete, false indicates that you are not yet done
        """
        return True

    def init_action(self):
        """
        Default init_action invoked by the CLI
        Generates config and ami_cache files
        Override in your subclass for custom initialization steps
        """
        self.generate_config()
        self.generate_ami_cache()

    def s3_prefix(self):
        """
        Allows subclasses to modify the default s3 prefix
        """
        return self.config.get('template').get('s3_prefix')

    def stack_outputs_directory(self):
        """
        Allows subclasses to modify the default stack outputs directory
        """
        return self.config.get('global').get('stack_outputs_directory') or 'stack_outputs'

    def _ensure_template_dir_exists(self):
        template_dir = self.s3_prefix()
        if not os.path.exists(template_dir):
            os.makedirs(template_dir)
        return template_dir

    @staticmethod
    def serialize_templates_helper(template, s3_client, s3_upload=True):

        # Create stack resources for template and all child templates
        raw_json = template.to_template_json()

        # Recursively iterate through each child template to serialize it and process its children
        for child, _, _, _, _ in template._child_templates:
            EnvironmentBase.serialize_templates_helper(
                template=child,
                s3_client=s3_client,
                s3_upload=s3_upload)

        if s3_upload:
            # Upload the template to the s3 bucket under the template_prefix
            s3_client.Bucket(Template.template_bucket_default).put_object(
                Key=template.resource_path,
                Body=raw_json,
                ACL=Template.upload_acl
            )

        # Save the template locally with the same file hierarchy as on s3
        with open(template.resource_path, 'w') as output_file:
            reloaded_template = json.loads(raw_json)
            output_file.write(json.dumps(reloaded_template, indent=4, separators=(',', ':')))

        print("Generated {} template".format(template.name))

        if s3_upload:
            print("S3:\t{}".format(utility.get_template_s3_url(Template.template_bucket_default, template.resource_path)))

        print("Local:\t{}\n".format(template.resource_path))

    def serialize_templates(self):
        s3_client = utility.get_boto_resource(self.config, 's3')
        self._ensure_template_dir_exists()

        s3_upload = self.config.get('template').get('s3_upload', True)

        EnvironmentBase.serialize_templates_helper(
            template=self.template,
            s3_client=s3_client,
            s3_upload=s3_upload)

    def estimate_cost(self, template_name=None, template_url=None, stack_params=None):
        cfn_conn = utility.get_boto_client(self.config, 'cloudformation')

        if not template_url:
            return None

        estimate_cost_url = cfn_conn.estimate_template_cost(
            TemplateURL=template_url,
            Parameters=stack_params)
        # else:
        #     template_body = self._load_template(template_name)
        #     estimate_cost_url = cfn_conn.estimate_template_cost(
        #         TemplateBody=template_body,
        #         Parameters=stack_params)

        return estimate_cost_url.get('Url')

    def _root_template_path(self):
        """
        Construct the root template resource path
        It never includes a timestamp because we need to find it by convention in the deploy step
        """
        return utility.get_template_s3_resource_path(
            prefix=self.s3_prefix(),
            template_name=self.globals.get('environment_name'),
            include_timestamp=False)

    def _root_template_url(self):
        """
        Construct the root template S3 URL
        """
        return utility.get_template_s3_url(
            bucket_name=self.template_args.get('s3_bucket'),
            resource_path=self._root_template_path())

    def create_action(self):
        """
        Default create_action invoked by the CLI
        Loads and validates config, initializes a new template instance, and writes it to file.
        Override the create_hook in your environment to inject all of your cloudformation resources
        """
        self.load_config()
        self.initialize_template()

        # Do custom troposphere resource creation in your overridden copy of this method
        self.create_hook()

        self.serialize_templates()

    def _ensure_stack_is_deployed(self, stack_name='UnnamedStack', sns_topic=None, stack_params=[]):
        """
        Deploys the root template to cloudformation using boto
        First attempts to issue an update stack command
        If this fails because the stack does not yet exist, then issues a create stack command
        """
        is_successful = False
        notification_arns = []

        if sns_topic:
            notification_arns.append(sns_topic.arn)

        template_url = self._root_template_url()

        cfn_conn = utility.get_boto_client(self.config, 'cloudformation')
        try:
            cfn_conn.update_stack(
                StackName=stack_name,
                TemplateURL=template_url,
                Parameters=stack_params,
                NotificationARNs=notification_arns,
                Capabilities=['CAPABILITY_IAM'])
            is_successful = True
            print("\nSuccessfully issued update stack command for %s\n" % stack_name)

        # Else stack doesn't currently exist, create a new stack
        except botocore.exceptions.ClientError as update_e:
            if "does not exist" in update_e.message:
                try:
                    cfn_conn.create_stack(
                        StackName=stack_name,
                        TemplateURL=template_url,
                        Parameters=stack_params,
                        NotificationARNs=notification_arns,
                        Capabilities=['CAPABILITY_IAM'],
                        DisableRollback=True,
                        TimeoutInMinutes=TIMEOUT)
                    is_successful = True
                    print("\nSuccessfully issued create stack command for %s\n" % stack_name)
                except botocore.exceptions.ClientError as create_e:
                    print("Deploy failed: \n\n%s\n" % create_e.message)
            else:
                raise

        return is_successful

    def add_parameter_binding(self, key, value):
        """
        Deployment parameters are used to provide values for parameterized templates

        The deploy_parameter_bindings is populated with hashes of the form:
         {
             'ParameterKey': <key>,
             'ParameterValue': <value>
         }

        :param key: String representing an input Parameter name in the root template
        :param value: Troposphere value for the Parameter
        """
        self.deploy_parameter_bindings.append({
            'ParameterKey': key,
            'ParameterValue': value
        })

    def deploy_action(self):
        """
        Default deploy_action invoked by the CLI.
        Loads and validates config, then deploys the root template to cloudformation using boto
        Override the deploy_hook in your environment to intercept the deployment process
        This can be useful for creating resources using boto outside of cloudformation
        """
        self.load_config()
        self.deploy_hook()

        stack_name = self.config['global']['environment_name']

        # initialize stack event monitor
        topic = None
        queue = None
        if self.stack_monitor and self.stack_monitor.has_handlers():
            (topic, queue) = self.stack_monitor.setup_stack_monitor(self.config)

        try:
            # First try to do an update-stack... if it doesn't exist, then try create-stack
            is_successful = self._ensure_stack_is_deployed(
                stack_name,
                sns_topic=topic,
                stack_params=self.deploy_parameter_bindings)

            if self.stack_monitor and is_successful:
                self.stack_monitor.start_stack_monitor(queue, stack_name, debug=self.globals['print_debug'])

        except KeyboardInterrupt:
            if self.stack_monitor:
                print('KeyboardInterrupt: calling cleanup')
                self.stack_monitor.cleanup_stack_monitor(topic, queue)
            raise

        if self.stack_monitor:
            self.stack_monitor.cleanup_stack_monitor(topic, queue)

    def delete_action(self):
        """
        Default delete_action invoked by CLI
        Loads and validates config, then issues the delete stack command to the root stack
        Override the delete_hook in your environment to intercept the delete process with your own code
        This can be useful for deleting any resources that were created outside of cloudformation
        """
        self.load_config()
        self.delete_hook()

        cfn_conn = utility.get_boto_client(self.config, 'cloudformation')
        stack_name = self.config['global']['environment_name']

        cfn_conn.delete_stack(StackName=stack_name)
        print("\nSuccessfully issued delete stack command for %s\n" % stack_name)

    def _validate_config_helper(self, schema, config, path):
        # Check each requirement
        for (req_key, req_value) in six.iteritems(schema):

            # Check for key match, usually only one match but parametrized keys can have multiple matches
            # Uses 'filename' match, similar to regex but only supports '?', '*', [XYZ], [!XYZ]
            filter_fun = lambda candidate_key: fnmatch(candidate_key, req_key)

            # Find all config keys matching the requirement
            matches = filter(filter_fun, config.keys())
            if not matches:
                message = "Config file missing section " + str(path) + ('.' if path is not '' else '') + req_key
                raise ValidationError(message)

            # Validate each matching config entry
            for matching_key in matches:
                new_path = path + ('.' if path is not '' else '') + matching_key

                # ------------ value check -----------
                if isinstance(req_value, six.string_types):
                    req_type = res.get_type(req_value)

                    if not isinstance(config[matching_key], req_type):
                        message = "Type mismatch in config, %s should be of type %s, not %s" % \
                                  (new_path, req_value, type(config[matching_key]).__name__)
                        raise ValidationError(message)
                    # else:
                    #     print("%s validated: %s == %s" % (new_path, req_value, type(config[matching_key]).__name__))

                # if the schema is nested another level .. we must go deeper
                elif isinstance(req_value, dict):
                    matching_value = config[matching_key]
                    if not isinstance(matching_value, dict):
                        message = "Type mismatch in config, %s should be a dict, not %s" % \
                                  (new_path, type(matching_value).__name__)
                        raise ValidationError(message)

                    self._validate_config_helper(req_value, matching_value, new_path)

                elif isinstance(req_value, list):
                    matching_value = config[matching_key]
                    if not isinstance(matching_value, list):
                        message = "Type mismatch in config, %s should be a list, not %s" % \
                                  (new_path, type(matching_value).__name__)
                        raise ValidationError(message)

    def _validate_region(self, config):
        """
        Checks boto.region_name against the list of valid regions raising an exception if not.
        """
        valid_regions = config['global']['valid_regions']
        region_name = config['boto']['region_name']
        if region_name not in valid_regions:
            raise ValidationError('Unrecognized region name: ' + region_name)

    def _validate_config(self, config, factory_schema=res.CONFIG_REQUIREMENTS):
        """
        Compares provided dict against TEMPLATE_REQUIREMENTS. Checks that required all sections and values are present
        and that the required types match. Throws ValidationError if not valid.
        :param config: dict to be validated
        """
        config_reqs_copy = copy.deepcopy(factory_schema)

        # Merge in any requirements provided by config handlers
        for handler in self._config_handlers:
            config_reqs_copy.update(handler.get_config_schema())

        self._validate_config_helper(config_reqs_copy, config, '')

        # # Validate region
        # self._validate_region(config)

    def _add_config_handler(self, handler):
        """
        Register classes that will augment the configuration defaults and/or validation logic here
        """

        if not hasattr(handler, 'get_factory_defaults') or not callable(getattr(handler, 'get_factory_defaults')):
            raise ValidationError('Class %s cannot be a config handler, missing get_factory_defaults()' % type(handler).__name__)

        if not hasattr(handler, 'get_config_schema') or not callable(getattr(handler, 'get_config_schema')):
            raise ValidationError('Class %s cannot be a config handler, missing get_config_schema()' % type(handler).__name__)

        self._config_handlers.append(handler)

    @staticmethod
    def _config_env_override(config, path, print_debug=False):
        """
        Update config value with values from the environment variables. If the environment variable exists
        the config value is replaced with its value.

        For config parameters like template.ec2_key_default this function will expect an environment
        variable matching the <section label>_<config_key> in all caps (e.g. TEMPLATE_EC2_KEY_DEFAULT).
        For environment variables containing multiple subsections the same pattern applies.

        For example: self._update_config_from_env('db', 'password') for the config file:
        {
            ...
            'db': {
                'label1': {
                    ...
                    'password': 'changeme'
                },
                'label2': {
                    ...
                    'password': 'changeme]'
                }
            }
        }

        Would replace those two database passwords if the following is run from the shell:
        > export DB_LABEL1_PASSWORD=myvoiceismypassword12345
        > export DB_LABEL2_PASSWORD=myvoiceismyotherpassword12345
        """
        for key, val in six.iteritems(config):
            new_path = path + ('.' if path is not '' else '') + key
            env_name = '_'.join(new_path.split('.')).upper()

            if not isinstance(val, dict):
                env_value = os.environ.get(env_name)
                if print_debug:
                    print("Checking %s (%s)" % (env_name, new_path))

                if env_value is None:
                    continue

                # TODO: Need better schema validation for non-string values from env vars

                # Convert true/false strings to booleans for schema validation
                if env_value.lower() == 'true':
                    env_value = True
                elif env_value.lower() == 'false':
                    env_value = False

                default_value = config.get(key)
                config[key] = env_value if env_value is not None else default_value

                if env_value is not None:
                    print("* Updating %s from '%s' to value of '%s'" % (new_path, default_value, env_name))

            else:
                EnvironmentBase._config_env_override(config[key], new_path, print_debug=print_debug)

    def generate_config(self):
        """
        Generate config dictionary from defaults
        Add defaults from all registered config handlers (added patterns, etc.)
        Write file to self.config_filename
        """

        if os.path.isfile(self.config_filename):
            overwrite = input("%s already exists. Overwrite? (y/n) " % self.config_filename).lower()
            print
            if not overwrite == 'y':
                return

        config = copy.deepcopy(res.FACTORY_DEFAULT_CONFIG)

        # Merge in any defaults provided by registered config handlers
        for handler in self._config_handlers:
            config.update(handler.get_factory_defaults())

        with open(self.config_filename, 'w') as f:
            f.write(json.dumps(config, indent=4, sort_keys=True, separators=(',', ': ')))
            print('Generated config file at %s\n' % self.config_filename)

    def load_config(self, view=None, config=None):
        """
        Load config from self.config_filename, break if it doesn't exist
        Load any overrides from environment variables
        Validate all loaded values
        """
        # Allow overriding the view for testing purposes
        if not view:
            view = self.view

        # Allow overriding of the entire config object
        if self.config_file_override:
            config = self.config_file_override

        # Else read from file
        else:
            config = res.load_file('', self.config_filename)

        # Load in cli config overrides
        view.update_config(config)

        # record value of the debug variable
        debug = config['global']['print_debug']

        # Check the environment variables for any overrides
        self._config_env_override(config, '', print_debug=debug)

        # Validate and save results
        self._validate_config(config)
        self.config = config

        # Save shortcut references to commonly referenced config sections
        self.globals = self.config.get('global', {})
        self.template_args = self.config.get('template', {})

        # Register all stack handlers
        if self.globals['monitor_stack']:
            self.stack_monitor = monitor.StackMonitor(self.globals['environment_name'])
            self.stack_monitor.add_handler(self)

    def initialize_template(self):
        """
        Create new Template instance, set description and common parameters and load AMI cache.
        """
        print('\nGenerating templates for {} stack\n'.format(self.globals['environment_name']))

        # Configure Template class with S3 settings from config
        Template.template_bucket_default = self.template_args.get('s3_bucket')
        Template.s3_path_prefix = self.s3_prefix()
        Template.stack_timeout = self.template_args.get("timeout_in_minutes")
        Template.upload_acl = self.template_args.get('s3_upload_acl')
        Template.include_timestamp = self.template_args.get('include_timestamp')

        Template.include_templateValidationHash_output = self.template_args.get('include_templateValidationHash_output')
        Template.include_dateGenerated_output = self.template_args.get('include_dateGenerated_output')

        # Create the root template object
        self.template = Template(self.globals.get('environment_name', 'default_template'))
        self.template.description = self.template_args.get('description', 'No Description Specified')
        self.template.resource_path = self._root_template_path()

        ec2_key = self.config.get('template').get('ec2_key_default', 'default-key')
        self.template._ec2_key = self.template.add_parameter(
            Parameter(
                'ec2Key',
                Type='String',
                Default=ec2_key,
                Description='Name of an existing EC2 KeyPair to enable SSH access to the instances',
                AllowedPattern=res.get_str('ec2_key'),
                MinLength=1,
                MaxLength=255,
                ConstraintDescription=res.get_str('ec2_key_message')
            )
        )

        bucket_name = self.config.get('logging').get('s3_bucket')

        self.template.add_utility_bucket(name=bucket_name)

        self.template.add_log_group()
        self.template.add_vpcflowlogs_role()

        ami_filename = self.config['template'].get('ami_map_file')
        if ami_filename:
            ami_cache = res.load_yaml_file(ami_filename)
            self.template.add_ami_mapping(ami_cache)

    def generate_ami_cache(self):
        """
        Generate ami_cache.json file from defaults
        """
        ami_cache_filename = res.DEFAULT_AMI_CACHE_FILENAME + res.EXTENSIONS[0]

        if os.path.isfile(ami_cache_filename):
            overwrite = input("%s already exists. Overwrite? (y/n) " % ami_cache_filename).lower()
            print
            if not overwrite == 'y':
                return

        with open(ami_cache_filename, 'w') as f:
            f.write(json.dumps(res.FACTORY_DEFAULT_AMI_CACHE, indent=4, separators=(',', ': ')))
            print("Generated AMI cache file at %s\n" % ami_cache_filename)

    def to_json(self):
        """
        Centralized method for outputting the root template with a timestamp identifying when it
        was generated and for creating a SHA256 hash representing the template for validation purposes
        Also recursively processess all child templates
        """
        return self.template.to_template_json()

    # Called after add_child_template() has attached common parameters and some instance attributes:
    # - RegionMap: Region to AMI map, allows template to be deployed in different regions without updating AMI ids
    # - ec2Key: keyname to use for ssh authentication
    # - vpcCidr: IP block claimed by whole VPC
    # - vpcId: resource id of VPC
    # - commonSecurityGroup: sg identifier for common allowed ports (22 in from VPC)
    # - utilityBucket: S3 bucket name used to send logs to
    # - [public|private]Subnet[0-9]: indexed and classified subnet identifiers
    #
    # and some instance attributes referencing the attached parameters:
    # - self.vpc_cidr
    # - self.vpc_id
    # - self.common_security_group
    # - self.utility_bucket
    # - self.subnets: keyed by type, layer, and AZ index (e.g. self.subnets['public']['web'][1])
    def add_child_template(self, child_template, merge=False, depends_on=[]):
        """
        Saves reference to provided template. References are processed in write_template_to_file().
        :param child_template: The Environmentbase Template you want to associate with the current instances
        :param depends_on: List of upstream resources that must be processes before the provided template
        :param merge: Determines whether the resource is attached as a child template or all of its resources merged
        into the current template
        """
        return self.template.add_child_template(child_template, merge=merge, depends_on=depends_on)

    def write_stack_outputs_to_file(self, event_data):
        """
        Given the stack event data, determine if the stack has finished executing (CREATE_COMPLETE or UPDATE_COMPLETE)
        If it has, write the stack outputs to file
        """
        if event_data['type'] == 'AWS::CloudFormation::Stack' and \
                (event_data['status'] == 'CREATE_COMPLETE' or event_data['status'] == 'UPDATE_COMPLETE'):
            self.write_stack_output_to_file(stack_id=event_data['id'], stack_name=event_data['name'])

    def write_stack_output_to_file(self, stack_id, stack_name):
        """
        Given a CFN stack's physical resource ID, query the stack for its outputs
        Save outputs to file as JSON at ./<stack_outputs_dir>/<stack_name>.json
        """
        # Grab all the outputs from the cfn stack object as k:v pairs
        stack_outputs = {}
        for output in self.get_cfn_stack_obj(stack_id).outputs:
            stack_outputs[output.key] = output.value

        stack_outputs_dir = self.stack_outputs_directory()

        # Ensure <stack_outputs_dir> directory exists
        if not os.path.isdir(stack_outputs_dir):
            os.mkdir(stack_outputs_dir)

        # Write the JSON-formatted stack outputs to ./<stack_outputs_dir>/<stack_name>.json
        stack_output_filename = os.path.join(stack_outputs_dir, stack_name + '.json')
        with open(stack_output_filename, 'w') as output_file:
            output_file.write(json.dumps(stack_outputs, indent=4, separators=(',', ':')))

        if self.globals['print_debug']:
            print("Outputs for {0} written to {1}\n".format(stack_name, stack_output_filename))

    def get_stack_output(self, stack_id, output_name):
        """
        Given the PhysicalResourceId of a Stack and a specific output key, return the output value
        Raise an exception if the output key is not found
        Example:
            def stack_event_hook(self, event_data):
                elb_dns_name = self.get_stack_output(event_data['id'], 'ElbDnsName')
        """
        stack_obj = self.get_cfn_stack_obj(stack_id)

        for output in stack_obj.outputs:
            if output.key == output_name:
                return output.value

        # If the output wasn't found in the stack, raise an exception
        raise Exception("%s did not output %s" % (stack_obj.stack_name, output_name))

    def get_cfn_stack_obj(self, stack_id):
        """
        Given the unique physical stack ID, return exactly one cloudformation stack object
        """
        return self.get_cfn_connection().describe_stacks(stack_id)[0]

    def get_cfn_connection(self):
        """
        We persist the CFN connection so that we don't create a new session with each request
        """
        if not self.cfn_connection:
            self.cfn_connection = cloudformation.connect_to_region(self.config.get('boto').get('region_name'))
        return self.cfn_connection

    def get_sts_credentials(self, role_session_name, role_arn):
        """
        We persist the STS credentials so that we don't create a new session with each request
        """
        if not self.sts_credentials:
            sts_connection = sts.STSConnection()
            assumed_role = sts_connection.assume_role(
                role_arn=role_arn,
                role_session_name=role_session_name
            )
            self.sts_credentials = assumed_role.credentials
        return self.sts_credentials



from environmentbase.networkbase import NetworkBase


class MyEnvClass(NetworkBase):
    """
    Class creates a VPC and common network components for the environment
    """

    def create_hook(self):
        # Do custom troposphere resource creation here
        super(MyEnvClass, self).create_hook()


if __name__ == '__main__':
    MyEnvClass()



#!/usr/bin/env python
"""
environmentbase

Tool bundle manages generation, deployment, and feedback of cloudformation resources.

Usage:
    environmentbase (init|create|deploy|delete) [--config-file <FILE_LOCATION>] [--debug] [--template-file=<TEMPLATE_FILE>]

Options:
  -h --help                            Show this screen.
  -v --version                         Show version.
  --debug                              Prints parent template to console out.
  --config-file <CONFIG_FILE>          Name of json configuration file. Default value is config.json
  --stack-name <STACK_NAME>            User-definable value for the CloudFormation stack being deployed.
  --template-file=<TEMPLATE_FILE>      Name of template to be either generated or deployed.
"""

from docopt import docopt
from . import version
import json


class CLI(object):

    def __init__(self, quiet=False, doc=__doc__):
        """
        CLI constructor is responsible for parsing sys.argv to collect configuration information.
        If you need to change the config file from the default name set the property 'config_filename'
        from the constructor.
        quiet: is provided to suppress output, primarily for unit testing
        """
        self.quiet = quiet

        self.args = docopt(doc, version='environmentbase %s' % version.__version__)

        # Parsing this config filename here is required since
        # the file is already loaded in self.update_config()
        self.config_filename = self.args.get('--config-file')

    def update_config(self, config):
        """
        The controller provides its config object containing settings loaded from file.  Potentially from the filename
        provided in the constructor above.  This function allows the CLI to override any of those values the user may
        have requested.
        """
        if self.args.get('--debug'):
            config['global']['print_debug'] = True

            if not self.quiet:
                print("CLI arguments %s" % json.dumps(self.args, indent=4, sort_keys=True))

        template_file = self.args.get('--template-file')
        if template_file is not None:
            config['global']['environment_name'] = template_file

    def _process_request_helper(self, controller):
        if self.args.get('init', False):
            controller.init_action()

        if self.args.get('create', False):
            controller.create_action()

        elif self.args.get('deploy', False):
            controller.deploy_action()

        elif self.args.get('delete', False):
            controller.delete_action()

    def process_request(self, controller):
        """
        Controller has finished initializing its config. This function maps user requested action to
        controller.XXX_action().  Currently supported actions: init_action(), create_action(), deploy_action(), delete_action().
        """
        print

        # Allow the full stack trace to print out when the debug flag is enabled
        if self.args.get('--debug'):
            self._process_request_helper(controller)
        # Otherwise catch it and print the error message
        else:
            try:
                self._process_request_helper(controller)
            except Exception as e:
                print("ERROR:\n\t{}\n\nTry running with the --debug flag\n".format(e.message))



from environmentbase.template import Template
import environmentbase.resources as res
from environmentbase.networkbase import NetworkBase
from environmentbase.environmentbase import EnvConfig
from troposphere import Ref, Parameter, GetAtt, Output, Join, rds, ec2


class RDS(Template):
    """
    Adds an RDS instance.
    """

    # default configuration values
    DEFAULT_CONFIG = {
        'db': {
            # Database label
            'mydb': {
                'db_instance_type_default': 'db.m1.small',
                'rds_user_name': 'defaultusername',
                # Actual database name, cannot include non-alphanumeric characters (e.g. '-')
                'master_db_name': 'mydb',
                'volume_size': 100,
                'backup_retention_period': 30,
                'rds_engine': 'MySQL',
                # 5.6.19 is no longer supported
                'rds_engine_version': '5.6.22',
                'preferred_backup_window': '02:00-02:30',
                'preferred_maintenance_window': 'sun:03:00-sun:04:00',
                # Name of vm snapshot to use, empty string ('') means don't use an old snapshot
                # Note: 'master_db_name' value will be overridden if snapshot_id is non-empty
                'snapshot_id': '',
                # Empty string is ignored, requiring manual parameter binding instead
                'password': 'changeme'
            }
        }
    }

    # schema of expected types for config values
    CONFIG_SCHEMA = {
        'db': {
            '*': {
                'db_instance_type_default': 'str',
                'rds_user_name': 'str',
                'master_db_name': 'str',
                'volume_size': 'int',
                'backup_retention_period': 'int',
                'rds_engine': 'str',
                'rds_engine_version': 'str',
                'preferred_backup_window': 'str',
                'preferred_maintenance_window': 'str',
                'snapshot_id': 'str',
                'password': 'str'
            }
        }
    }

    def __init__(self,
                 tier_name,
                 connect_from_cidr=None,
                 connect_from_sg=None,
                 subnet_set='private',
                 config_map=DEFAULT_CONFIG['db']):
        """
        Method initializes host in a given environment deployment
        @param tier_name: [string] - name of the tier to assign
        @param connect_from_cidr: [Ref|string] - CIDR notation for external access to RDS instance. Cannot be used in conjunction with
        connect_from_sg. If neither setting is used the CIDR of the VPC will be used.
        @param connect_from_sg: [Ref|string] - Name of security group allowed to access RDS instance. Cannot be used in conjunction with
        connect_from_cidr.
        @param subnet_set: 'public' or 'private', Type of subnets to use
        @param config_map: map of database config settings to be deployed.
        """

        self.tier_name = tier_name
        self.config_map = config_map
        self.subnet_set = subnet_set

        if connect_from_cidr and connect_from_sg:
            raise ValueError("RDS instance cannot be configured to be accessible from both CIDR and security group.")

        self.connect_from_cidr = connect_from_cidr
        self.connect_from_sg = connect_from_sg

        self.data = {}

        super(RDS, self).__init__(template_name=tier_name + 'RDSInstance')

    # When no config.json file exists a new one is created using the 'factory default' file.  This function
    # augments the factory default before it is written to file with the config values required
    @staticmethod
    def get_factory_defaults():
        return RDS.DEFAULT_CONFIG

    # When the user request to 'create' a new RDS template the config.json file is read in. This file is checked to
    # ensure all required values are present. Because RDS has additional requirements beyond that of
    # EnvironmentBase this function is used to add additional validation checks.
    @staticmethod
    def get_config_schema():
        return RDS.CONFIG_SCHEMA

    def add_parameters(self, db_label, db_config):
        instance_type_param = self.add_parameter(Parameter(
            db_label.lower() + self.tier_name.title() + 'RdsInstanceType',
            Default=db_config.get('db_instance_type_default'),
            Type='String',
            Description='DB Instance Type for the RDS instance.',
            AllowedValues=res.COMMON_STRINGS.get('valid_db_instance_types'),
            ConstraintDescription=res.COMMON_STRINGS.get('valid_db_instance_type_message')))

        name_param = self.add_parameter(Parameter(
            db_label.lower() + self.tier_name.title() + 'RdsDbName',
            Type='String',
            Default=db_config.get('master_db_name'),
            Description='Master RDS database name for the RDS instance.',
            MinLength=3,
            MaxLength=32,
            ConstraintDescription='must be 3 or more characters and not longer than 64 characters.'))

        user_name_param = self.add_parameter(Parameter(
            db_label.lower() + self.tier_name.title() + 'RdsUserName',
            Default=db_config.get('rds_user_name'),
            Type='String',
            Description='Master RDS User name for the RDS instance',
            MinLength=3,
            MaxLength=64,
            ConstraintDescription='must be 3 or more characters and not longer than 64 characters.'))

        user_password_param = self.add_parameter(Parameter(
            db_label.lower() + self.tier_name.title() + 'RdsMasterUserPassword',
            NoEcho=True,
            Type='String',
            Description='Master RDS User Password for the RDS instance.',
            MinLength=12,
            MaxLength=64,
            ConstraintDescription='must be 12 or more characters and not longer than 64 characters.'))

        return instance_type_param, name_param, user_name_param, user_password_param

    # Called after add_child_template() has attached common parameters and some instance attributes:
    # - RegionMap: Region to AMI map, allows template to be deployed in different regions without updating AMI ids
    # - ec2Key: keyname to use for ssh authentication
    # - vpcCidr: IP block claimed by whole VPC
    # - vpcId: resource id of VPC
    # - commonSecurityGroup: sg identifier for common allowed ports (22 in from VPC)
    # - utilityBucket: S3 bucket name used to send logs to
    # - [public|private]Subnet[0-9]: indexed and classified subnet identifiers
    #
    # and some instance attributes referencing the attached parameters:
    # - self.vpc_cidr
    # - self.vpc_id
    # - self.common_security_group
    # - self.utility_bucket
    # - self.subnets: keyed by type and index (e.g. self.subnets['public'][1])
    def build_hook(self):

        for db_label, db_config in self.config_map.iteritems():

            (db_instance_type,
             db_name,
             db_user_name,
             db_user_password) = self.add_parameters(db_label, db_config)

            subnet_type = self.get_subnet_type(self.subnet_set)
            subnet_group = self.add_resource(rds.DBSubnetGroup(
                db_label.lower() + 'RdsSubnetGroup',
                DBSubnetGroupDescription='Subnet group for the RDS instance',
                SubnetIds=self.subnets[subnet_type][self.subnet_set]))

            rds_sg = self.add_resource(
                ec2.SecurityGroup(
                    db_label.lower() + self.tier_name.title() + 'RdsSg',
                    GroupDescription='Security group for %s RDS tier' % self.tier_name.lower(),
                    VpcId=self.vpc_id)
            )

            rds_instance = self.add_resource(rds.DBInstance(
                db_label.lower() + self.tier_name.title() + 'RdsInstance',
                AllocatedStorage=db_config.get('volume_size', '100'),
                BackupRetentionPeriod=db_config.get('backup_retention_period', '30'),
                DBInstanceClass=Ref(db_instance_type),
                DBName=Ref(db_name),
                VPCSecurityGroups=[Ref(rds_sg)],
                DBSubnetGroupName=Ref(subnet_group),
                Engine=db_config.get('rds_engine', 'mysql'),
                EngineVersion=db_config.get('rds_engine_version', '5.6.19'),
                MasterUsername=Ref(db_user_name),
                MasterUserPassword=Ref(db_user_password),
                PreferredBackupWindow=db_config.get('preferred_backup_window', '02:00-02:30'),
                PreferredMaintenanceWindow=db_config.get('preferred_maintenance_window', 'sun:03:00-sun:04:00'),
                MultiAZ=True))

            # Set the snapshot id if provided (and null out the db name to avoid cfn error)
            if db_config['snapshot_id']:
                rds_instance.DBSnapshotIdentifier = db_config['snapshot_id']
                # DBName must be null when restoring from snapshot
                rds_instance.DBName = ''

            # Create the sg ingress rule for whatever port the rds instance needs
            ingress_rule = ec2.SecurityGroupIngress(
                db_label.lower() + self.tier_name.title() + 'RdsIngressRule',
                FromPort=GetAtt(rds_instance, "Endpoint.Port"),
                ToPort=GetAtt(rds_instance, "Endpoint.Port"),
                IpProtocol='tcp',
                GroupId=Ref(rds_sg))

            # Set the allowed origin on the ingress rule according the requested connect_from setting
            # OR vpc_cidr in no other setting provided
            if self.connect_from_sg:
                ingress_rule.SourceSecurityGroupId = self.connect_from_sg
            elif self.connect_from_cidr:
                ingress_rule.CidrIp = self.connect_from_cidr
            else:
                ingress_rule.CidrIp = self.vpc_cidr

            self.add_resource(ingress_rule)

            # Add the connection endpoint output
            self.add_output(Output(
                db_label.lower() + self.tier_name.title() + 'RdsEndpoint',
                Value=Join('', [
                    Ref(db_user_name), '@',
                    GetAtt(rds_instance, "Endpoint.Address"), ':',
                    GetAtt(rds_instance, "Endpoint.Port")])
            ))

            # Add the connection address output
            self.add_output(Output(
                db_label.lower() + self.tier_name.title() + 'RdsAddress',
                Value=GetAtt(rds_instance, "Endpoint.Address"))
            )

            self.data[db_label] = {
                'rds': rds_instance,
                'dbname': Ref(db_name),
                'endpoint_address': GetAtt(rds_instance, 'Endpoint.Address'),
                'endpoint_port': GetAtt(rds_instance, 'Endpoint.Port'),
                'masteruser': Ref(db_user_name),
                'masterpassword': Ref(db_user_password),
                'securitygroup': rds_sg
            }


class Controller(NetworkBase):
    """
    Example class showing how to use the RDS pattern file to generate an RDS database as a child stack

    To connect to this database log into the bastion and run:
    > sudo yum install -y mysql
    > mysql -h <db endpoint> -P <db port> -u <db username> -p
    """

    def create_hook(self):
        # Create the rds instance pattern (includes standard standard parameters)
        my_db = RDS(
            'dbTier',
            subnet_set=self.template.subnets['private'].keys()[0],
            config_map=db_config)

        # Attach pattern as a child template
        self.add_child_template(my_db)

    def deploy_hook(self):
        for db_label, db_config in self.config['db'].iteritems():
            db_resource_name = db_label.lower() + 'dbTier'.title() + 'RdsMasterUserPassword'
            print("adding %s" % db_resource_name)
            self.add_parameter_binding(key=db_resource_name, value=db_config['password'])


if __name__ == '__main__':

    db_config = {
        'label1': {
            'db_instance_type_default': 'db.m1.small',
            'rds_user_name': 'defaultusername',
            # Actual database name, cannot include non-alphanumeric characters (e.g. '-')
            'master_db_name': 'mydb',
            'volume_size': 100,
            'backup_retention_period': 30,
            'rds_engine': 'MySQL',
            # 5.6.19 is no longer supported
            'rds_engine_version': '5.6.22',
            'preferred_backup_window': '02:00-02:30',
            'preferred_maintenance_window': 'sun:03:00-sun:04:00',
            # Name of vm snapshot to use, empty string ('') means don't use an old snapshot
            # Note: 'master_db_name' value will be overridden if snapshot_id is non-empty
            'snapshot_id': '',
            'password': 'changeme111111111111'
        },
        'label2': {
            'db_instance_type_default': 'db.m1.small',
            'rds_user_name': 'defaultusername',
            # Actual database name, cannot include non-alphanumeric characters (e.g. '-')
            'master_db_name': 'mydb2',
            'volume_size': 100,
            'backup_retention_period': 30,
            'rds_engine': 'MySQL',
            # 5.6.19 is no longer supported
            'rds_engine_version': '5.6.22',
            'preferred_backup_window': '02:00-02:30',
            'preferred_maintenance_window': 'sun:03:00-sun:04:00',
            # Name of vm snapshot to use, empty string ('') means don't use an old snapshot
            # Note: 'master_db_name' value will be overridden if snapshot_id is non-empty
            'snapshot_id': '',
            'password': 'changeme1111111111111'
        }
    }

    my_config = res.FACTORY_DEFAULT_CONFIG
    my_config['db'] = db_config

    env_config = EnvConfig(config_handlers=[RDS])
    Controller(env_config=env_config, config_file_override=my_config)



from environmentbase.networkbase import NetworkBase
from environmentbase.patterns.bastion import Bastion
from environmentbase.patterns.ha_cluster import HaCluster


class MyEnvClass(NetworkBase):
    """
    Class creates a VPC and common network components for the environment
    """

    def create_hook(self):

        # Do custom troposphere resource creation here
        super(MyEnvClass, self).create_hook()

        # Add a bastion host as a child stack in the environment
        self.add_child_template(Bastion())

        # Add a preconfigured ASG + ELB as a child stack in the environment
        self.add_child_template(HaCluster(
            name="MyCluster",
            min_size=2, max_size=3,
            instance_type='t2.micro'))


if __name__ == '__main__':
    MyEnvClass()



from unittest2 import TestCase, main
import mock
from mock import patch
import os
import shutil
import sys
from tempfile import mkdtemp
from environmentbase import cli, template, utility
import troposphere as tropo
from troposphere import ec2
import yaml


class TemplateTestCase(TestCase):

    def setUp(self):
        # Change to a temp dir so auto generated file don't clutter the os
        self.temp_dir = mkdtemp()
        os.chdir(self.temp_dir)

    def tearDown(self):
        # Delete any files left in the temp dir
        shutil.rmtree(self.temp_dir)
        assert not os.path.isdir(self.temp_dir)

    def fake_cli(self, extra_args):
        args = ['environmentbase']
        args.extend(extra_args)

        with patch.object(sys, 'argv', args):
            my_cli = cli.CLI(quiet=True)
            my_cli.process_request = mock.MagicMock()

        return my_cli

    def _create_local_file(self, name, content):
        f = open(os.path.join(self.temp_dir, name), 'a')
        f.write(content)
        f.flush()
        return f

    def test_tropo_to_string(self):
        utility.tropo_to_string(tropo.Template())
        utility.tropo_to_string(tropo.Base64('efsdfsdf'))
        utility.tropo_to_string(tropo.Output('efsdfsdf', Value='dsfsdfs'))
        utility.tropo_to_string(tropo.Parameter('efsdfsdf', Type='dsfsdfs'))

        # These constructors recursively call themselves for some reason
        # Don't instantiate directly
        # utility.tropo_to_string(tropo.AWSProperty())
        # utility.tropo_to_string(tropo.AWSAttribute())

        utility.tropo_to_string(ec2.Instance(
            "ec2instance",
            InstanceType="m3.medium",
            ImageId="ami-951945d0"))

    def test_build_bootstrap(self):
        file1_name = 'arbitrary_file.txt'
        file1_content = 'line1\nline2\nline3'
        self._create_local_file(file1_name, file1_content)

        # Basic test
        template_snippet = template.Template.build_bootstrap([file1_name], prepend_line='')
        generated_json = yaml.load(utility.tropo_to_string(template_snippet))
        expected_json_1 = {"Fn::Base64": {"Fn::Join": ["\n", ["line1", "line2", "line3"]]}}
        self.assertEqual(generated_json, expected_json_1)

        # Advanced test

        # resources can't be accessed as files directly
        # lines starting with #~ are removed automatically
        file2_content = '#~this_line_should_be_stripped_out\nline4\nline5\nline6'

        # you can provided multiple files or content (mix and match) in the specified order
        # you can set the shabang to whatever you want
        # you can reference variables in the file content and set there values using variable_declarations
        # finally to can do any append cleanup commands to the bottom of the file with cleanup_commands
        template_snippet = template.Template.build_bootstrap(
            [file1_name, file2_content],
            prepend_line='#!/bin/fakesh',
            variable_declarations=['var_dec_line_1', 'var_dec_line_2'],
            cleanup_commands=['cleanup_line_1', 'cleanup_line_2'])

        generated_json = yaml.load(utility.tropo_to_string(template_snippet))
        expected_json_2 = {
            "Fn::Base64": {"Fn::Join": [
                "\n", [
                    "#!/bin/fakesh",
                    'var_dec_line_1', 'var_dec_line_2',
                    "line1", "line2", "line3",
                    "line4", "line5", "line6",
                    'cleanup_line_1', 'cleanup_line_2'
                ]
            ]}}

        self.assertEqual(generated_json, expected_json_2)

if __name__ == '__main__':
    main()
